<?php

/**
 * Plugin Name: SpamJam
 * Plugin URI: https://utopique.net/products/spamjam/
 * Description: SpamJam silently kills spam in comments and registration.
 * Version: 0.4.0
 * Author: Utopique
 * Author URI: https://utopique.net/
 * Copyright: 2022-2023 Utopique
 * Text Domain: spamjam
 * License: GPLv2 or later
 * Requires at least: 5.2
 * Tested up to: 6.3.0
 * Requires PHP: 7.0
 * WC requires at least: 3.3
 * WC tested up to: 7.8.1
 * PHP version 7
 *
 * @category        SpamJam
 * @package         SpamJam
 * @author          Utopique <support@utopique.net>
 * @license         GPL https://utopique.net
 * @link            https://utopique.net
 */
namespace SpamJam;

\defined( 'ABSPATH' ) or exit;

if ( function_exists( __NAMESPACE__ . '\\spamjam_fs' ) ) {
    spamjam_fs()->set_basename( false, __FILE__ );
} else {
    
    if ( !function_exists( __NAMESPACE__ . '\\spamjam_fs' ) ) {
        // Create a helper function for easy SDK access.
        function spamjam_fs()
        {
            global  $spamjam_fs ;
            
            if ( !isset( $spamjam_fs ) ) {
                // Include Freemius SDK.
                require_once dirname( __FILE__ ) . '/vendor/freemius/wordpress-sdk/start.php';
                $spamjam_fs = fs_dynamic_init( array(
                    'id'             => '11800',
                    'slug'           => 'spamjam',
                    'premium_slug'   => 'spamjam-pro',
                    'type'           => 'plugin',
                    'public_key'     => 'pk_a92dd1a28c55ffae9a6b997e98167',
                    'is_premium'     => false,
                    'premium_suffix' => 'Pro',
                    'has_addons'     => false,
                    'has_paid_plans' => true,
                    'menu'           => array(
                    'slug'    => 'spamjam',
                    'support' => false,
                ),
                    'is_live'        => true,
                ) );
            }
            
            return $spamjam_fs;
        }
        
        // Init Freemius.
        spamjam_fs();
        // Signal that SDK was initiated.
        do_action( 'spamjam_fs_loaded' );
    }
    
    /**
     * Load plugin translations
     *
     * @return translation
     */
    function SpamJam_Load_textdomain()
    {
        load_plugin_textdomain( 'spamjam', false, basename( dirname( __FILE__ ) ) . '/languages/' );
    }
    
    add_action( 'plugins_loaded', __NAMESPACE__ . '\\SpamJam_Load_textdomain' );
    /**
     * Block direct requests to wp-comments-post.php
     *
     * @return bool
     */
    function SpamJam_Block_referer()
    {
        if ( $referrer_url !== false || strpos( $referrer_url, get_site_url() ) !== false ) {
            return false;
        }
        return true;
    }
    
    /**
     * Comment blocklist
     *
     * @param string $comment the comment
     *
     * @return bool
     */
    function SpamJam_blocklist( $comment )
    {
        $blocklist = [
            '[url=',
            'replica',
            'pill',
            'viagra',
            'cialis',
            'drug',
            'medicine',
            'prescription'
        ];
        foreach ( $blocklist as $string ) {
            if ( stripos( $comment, $string ) !== false ) {
                return true;
            }
        }
        return false;
    }
    
    /**
     * Add Comment honeypot fields
     *
     * @param html $comment_field html for $comment_field
     *
     * @return html
     */
    function SpamJam_Comment_honeypot( $comment_field )
    {
        ?>
    <style>
    p.required label { clear: both; display: block; width: 100%;}
    p.required label::before {content: "⚠️ <?php 
        _e( 'Do not ', 'spamjam' );
        ?> ";}
    </style>
        <p style="display: none;" class="required">
        <label for="email_confirm">
            <?php 
        _e( 'confirm email:', 'spamjam' );
        ?>
        </label>
        <input type="email" name="email_confirm" id="email_confirm" size="30" />
        <?php 
        wp_nonce_field( 'spamjam_comment_nonce', 'spamjam_comment_nonce' );
        ?>
        <label for="comment">
            <?php 
        _e( 'fill comment:', 'spamjam' );
        ?>
        </label>
        <textarea id="comment" name="comment"></textarea>
        </p>
    <?php 
        return $comment_field;
    }
    
    /**
     * Note: the hook 'comment_form_after_fields' only targets
     * logged-out users.
     * Since we want to apply the security fields to all users, we'll use
     * 'comment_form_field_comment' (otherwise logged-in users get spamjammed)
     * so that the new fields are displayed before the submit button
     */
    add_action( 'comment_form_field_comment', __NAMESPACE__ . '\\SpamJam_Comment_honeypot', 100 );
    /**
     * Add query string to comment form action
     *
     * @return mixed
     */
    function SpamJam_Comment_Action_On_scroll()
    {
        // Check if Comment is enabled
        
        if ( is_singular() && comments_open() ) {
            /**
             * Create a seed
             */
            $comments_post = home_url( 'wp-comments-post.php' );
            $md5 = md5( $comments_post );
            $action_url = esc_url( add_query_arg( 'sj5', $md5, $comments_post ) );
            /**
             * Detect our comment form
             * Change commentForm.action when the scroll event triggers
             */
            echo  <<<EOT
<script>let commentForm = document.querySelector("#commentform, #ast-commentform, #ht-commentform");
document.onscroll = function () { commentForm.action = '{$action_url}'; };</script>
EOT
 ;
        }
    
    }
    
    add_action( 'wp_footer', __NAMESPACE__ . '\\SpamJam_Comment_Action_On_scroll', 99 );
    /**
     * Add custom field ID and name in lieu of the default "comment"
     *
     * @param mixed $field comment field
     *
     * @return html
     */
    function SpamJam_CommentField_replace( $field )
    {
        $replace = str_replace( '="comment"', '="sj-comment"', $field );
        return $replace;
    }
    
    add_filter( 'comment_form_field_comment', __NAMESPACE__ . '\\SpamJam_CommentField_replace' );
    /**
     * Get client IP address
     *
     * @return $ip IP
     */
    function SpamJam_Get_Client_ip()
    {
        $ip = '';
        $proxy_headers = [
            'HTTP_CF_CONNECTING_IP',
            // CloudFlare
            'HTTP_INCAP_CLIENT_IP',
            // Incapsula
            'HTTP_X_SUCURI_CLIENTIP',
            // Sucuri
            'HTTP_CLIENT_IP',
            // Client IP
            'HTTP_X_FORWARDED_FOR',
            // Any Proxy
            'REMOTE_ADDR',
        ];
        // Check for alternate headers indicating a forwarded IP address
        foreach ( $proxy_headers as $proxy_header ) {
            
            if ( isset( $_SERVER[$proxy_header] ) ) {
                $forwarded_ips = explode( ',', filter_var( $_SERVER[$proxy_header], FILTER_VALIDATE_IP ) );
                $forwarded_ip = array_shift( $forwarded_ips );
                
                if ( $forwarded_ip ) {
                    $ip = $forwarded_ip;
                    break;
                }
            
            }
        
        }
        return $ip;
    }
    
    /**
     * Run all SpamJam tests
     *
     * @param [type] $data comment data
     *
     * @return $data
     */
    function SpamJam_blocker( $data )
    {
        // Get client IP
        $ip = SpamJam_Get_Client_ip();
        // set spam_content $_POST['sj-comment'] as spam entry
        $data['spam_content'] = wp_kses_data( $_POST['sj-comment'] );
        // add md5 to array
        $data['sj5'] = wp_kses_data( $_POST['sj5'] );
        // debug
        /*
            echo '<pre>';
            print_r($data);
            foreach ($_POST as $key => $value) {
                echo "{$key} = {$value}\r\n";
            }
            die;
        */
        /**
         * Run all tests
         */
        // set $msg to empty value
        $msg = '';
        // Honeypot has been filled
        if ( !empty($_POST['email_confirm']) ) {
            $msg .= '460 ';
        }
        // Extra field has been filled
        if ( !empty($data['spam_content']) ) {
            $msg .= '461 ';
        }
        // Referer is not correct
        if ( SpamJam_Block_referer() ) {
            $msg .= '462 ';
        }
        // Comment content matches blocklist
        if ( SpamJam_blocklist( $data['comment_content'] ) ) {
            $msg .= '463 ';
        }
        // Comment nonce does not exist
        if ( !isset( $_POST['spamjam_comment_nonce'] ) ) {
            $msg .= '464 ';
        }
        // Comment nonce is incorrect
        if ( !wp_verify_nonce( $_POST['spamjam_comment_nonce'], 'spamjam_comment_nonce' ) ) {
            $msg .= '465 ';
        }
        // MD5 is not set
        if ( empty($data['sj5']) ) {
            $msg .= '466 ';
        }
        // MD5 does not match
        if ( $data['sj5'] != md5( home_url( 'wp-comments-post.php' ) ) ) {
            $msg .= '467 ';
        }
        // if errors exist, exit
        
        if ( !empty($msg) ) {
            $msg = str_replace( ' ', ', ', trim( $msg ) );
            $msg_count = substr_count( $msg, ',' );
            $error_count = ( $msg_count > 0 ? 'Errors' : 'Error' );
            $msg = sprintf( __( "<p><strong>%s: </strong>%s. You've been SpamJammed!</p>", 'spamjam' ), $error_count, $msg );
            wp_die( $msg );
        }
        
        // store $_POST['sj-comment'] in $comment
        //$comment = wp_kses_data($_POST['sj-comment']);
        // store $_POST['comment'] in $data['sj-comment']
        //$data['sj-comment'] = wp_kses_data($_POST['comment']);
        // set $data['comment_content'] with $_POST['sj-comment']
        //$data['comment_content'] = $comment;
        //echo '<pre>'; print_r($data); die;
        return $data;
    }
    
    // Process rules on the frontend only
    if ( !is_admin() ) {
        add_filter( 'preprocess_comment', __NAMESPACE__ . '\\SpamJam_blocker' );
    }
    /**
     * Swap our custom comment field to WP default
     *
     * @param mixed $field comment field
     *
     * @return void
     */
    function SpamJam_CommentField_swap( $field )
    {
        
        if ( basename( $_SERVER['PHP_SELF'] ) === 'wp-comments-post.php' && isset( $_POST ) && isset( $_POST['comment'] ) && isset( $_POST['sj-comment'] ) && isset( $_GET['sj5'] ) ) {
            $comment = wp_kses_data( $_POST['sj-comment'] );
            $_POST['sj-comment'] = wp_kses_data( $_POST['comment'] );
            $_POST['comment'] = $comment;
            $_POST['sj5'] = wp_kses_data( $_GET['sj5'] );
        }
    
    }
    
    add_action( 'init', __NAMESPACE__ . '\\SpamJam_CommentField_swap' );
    /**
     * Add admin settings page and menu
     */
    require_once plugin_dir_path( __FILE__ ) . 'includes/admin.php';
}

// end spamjam_fs